<?php
/*
*	<id>chenzhen:snowflakesSMF</id>
*	<name>Snowflakes</name>
*	<type>modification</type>
*/

/*
 * Snowflakes display Modification for SMF 2.1 forums c/o Chen Zhen @ https://web-develop.ca
 * Copyright 2014 - 2023
 * This software package is distributed under the GNU Lesser General Public License version 3 ( https://opensource.org/licenses/lgpl-3.0.html )
*/

if (file_exists(dirname(__FILE__) . '/SSI.php') && !defined('SMF'))
	require_once(dirname(__FILE__) . '/SSI.php');
elseif (!defined('SMF'))
	die('<b>Error:</b> Cannot install - please verify you put this in the same place as SMF\'s SSI.php.');

function smf_snowflakes_admin(&$admin_areas)
{
	global $txt;
	if (!defined("SNOW_FLAKES_IS_MOBILE") || !defined("SNOW_FLAKES_IS_TABLET"))
		smf_snowflakes_helpText();

	loadLanguage('snowflakesSMF');

	$snowflakesSMF = array(
		'snowflakes' => array($txt['smf_snowflakes_title']),
	);

	$admin_areas['config']['areas']['modsettings']['subsections'] += $snowflakesSMF;

}

function smf_snowflakes_mod(&$subActions)
{
	global $txt;
	loadLanguage('snowflakesSMF');

	$subActions['snowflakes'] = 'smf_snowflakes_settings';
}

function smf_snowflakes_scripts()
{
	global $txt;
	loadLanguage('snowflakesSMF');
	$scripts = array();

	for ($i=0; $i<99; $i++)
	{
		if (empty($txt['smf_snowflakes' . $i]))
			break;

		$scripts[$i] = $txt['smf_snowflakes' . $i];
	}

	$scripts[99] = $txt['smf_snowflakes99'];

	return $scripts;
}

function smf_snowflakes_settings($return_config = false)
{
	global $txt, $helptxt, $context, $scripturl, $smcFunc, $settings;

	// load language files
	loadLanguage('snowflakesSMF');

	if (!allowedTo('admin_forum'))
		fatal_lang_error('snowflakesSMF_ErrorMessage', false);

	$scripts = smf_snowflakes_scripts();
	list($imageTypeDefault, $imagePath, $imageTypeDefault[99], $imagePath[99]) = array(array(), array(), '', '');
	$snowImages = array(2, 5, 8);
	$noOptions = array(4, 5);
	@ini_set('memory_limit', '128M');
	$snowflakesSettings = smf_snowflakes_getSettings();
	$type = isset($snowflakesSettings['snowflakesType']) ? abs($snowflakesSettings['snowflakesType']) : 0;
	$snowflakesSettings['snowflakesType'] = $type > (count($scripts) - 2) ? 99 : $type;
	$type = $snowflakesSettings['snowflakesType'];
	$snowflakesActions = !empty($snowflakesSettings['snowflakesActions']) ? preg_replace('/[^A-Za-z0-9,_~|]/', '', $snowflakesSettings['snowflakesActions']) : '';
	$snowflakesDisableIds = !empty($snowflakesSettings['snowflakesDisableIds']) ? preg_replace('/[^A-Za-z0-9,_]/', '', $snowflakesSettings['snowflakesDisableIds']) : '';
	$snowflakesDisableClasses = !empty($snowflakesSettings['snowflakesDisableClasses']) ? preg_replace('/[^A-Za-z0-9,_]/', '', $snowflakesSettings['snowflakesDisableClasses']) : '';
	$snowflakesDisableRange = !empty($snowflakesSettings['snowflakesDisableRange']) ? intval($snowflakesSettings['snowflakesDisableRange']) : 0;

	// default settings
	foreach ($scripts as $i => $script)
	{
		$imageTypeDefault[$i] = $i == 8 ? 'wired_lights1.gif' : 'snowflake.png';
		$imagePath[$i] = $i == 8 ? 'wired_lights' : 'snowflakes';
		$snowflakesSettings['snowflakesMinSize' . $i] = isset($snowflakesSettings['snowflakesMinSize' . $i]) ? abs($snowflakesSettings['snowflakesMinSize' . $i]) : 5;
		$snowflakesSettings['snowflakesMaxSize' . $i] = isset($snowflakesSettings['snowflakesMaxSize' . $i]) ? abs($snowflakesSettings['snowflakesMaxSize' . $i]) : 50;
		$snowflakesSettings['snowflakesNewOn' . $i] = isset($snowflakesSettings['snowflakesNewOn' . $i]) ? abs($snowflakesSettings['snowflakesNewOn' . $i]) : 1000;
		$snowflakesSettings['snowflakesFlakeColor' . $i] = isset($snowflakesSettings['snowflakesFlakeColor' . $i]) ? $snowflakesSettings['snowflakesFlakeColor' . $i] : '#0099FF';
		$snowflakesSettings['snowflakesMinSpeed' . $i] = isset($snowflakesSettings['snowflakesMinSpeed' . $i]) ? abs($snowflakesSettings['snowflakesMinSpeed' . $i]) : 1;
		$snowflakesSettings['snowflakesMaxSpeed' . $i] = isset($snowflakesSettings['snowflakesMaxSpeed' . $i]) ? abs($snowflakesSettings['snowflakesMaxSpeed' . $i]) : 5;
		$snowflakesSettings['snowflakesCollection' . $i] = isset($snowflakesSettings['snowflakesCollection' . $i]) ? $snowflakesSettings['snowflakesCollection' . $i] : '';
		$snowflakesSettings['snowflakesMobile' . $i] = isset($snowflakesSettings['snowflakesMobile' . $i]) ? abs($snowflakesSettings['snowflakesMobile' . $i]) : 1;
		$snowflakesSettings['snowflakesFlicker' . $i] = isset($snowflakesSettings['snowflakesFlicker' . $i]) ? abs($snowflakesSettings['snowflakesFlicker' . $i]) : 0;
		$snowflakesSettings['snowflakesWidth' . $i] = isset($snowflakesSettings['snowflakesWidth' . $i]) ? abs($snowflakesSettings['snowflakesWidth' . $i]) : 35;
		$snowflakesSettings['snowflakesHeight' . $i] = isset($snowflakesSettings['snowflakesHeight' . $i]) ? abs($snowflakesSettings['snowflakesHeight' . $i]) : 20;
		$snowflakesSettings['snowflakesTimeMin' . $i] = isset($snowflakesSettings['snowflakesTimeMin' . $i]) ? abs($snowflakesSettings['snowflakesTimeMin' . $i]) : 400;
		$snowflakesSettings['snowflakesTimeMax' . $i] = isset($snowflakesSettings['snowflakesTimeMax' . $i]) ? abs($snowflakesSettings['snowflakesTimeMax' . $i]) : 1200;
		$snowflakesSettings['snowflakesExplodeTimeMin' . $i] = isset($snowflakesSettings['snowflakesExplodeTimeMin' . $i]) ? abs($snowflakesSettings['snowflakesExplodeTimeMin' . $i]) : 400;
		$snowflakesSettings['snowflakesExplodeTimeMax' . $i] = isset($snowflakesSettings['snowflakesExplodeTimeMax' . $i]) ? abs($snowflakesSettings['snowflakesExplodeTimeMax' . $i]) : 800;
		$snowflakesSettings['snowflakesImage' . $i] = isset($snowflakesSettings['snowflakesImage' . $i]) ? $snowflakesSettings['snowflakesImage' . $i] : $imageTypeDefault[$i];
		$snowflakesSettings['snowflakesOffset' . $i] = isset($snowflakesSettings['snowflakesOffset' . $i]) ? (float)$snowflakesSettings['snowflakesOffset' . $i] : 0;
		$snowflakesSettings['snowflakesContainerHeight' . $i] = isset($snowflakesSettings['snowflakesContainerHeight' . $i]) ? abs($snowflakesSettings['snowflakesContainerHeight' . $i]) : 4.5;
	}

	$context['snowflakes_images'] = smf_snowflakes_images($snowflakesSettings['snowflakesImage' . $snowflakesSettings['snowflakesType']], $snowflakesSettings['snowflakesType']);
	$config_vars = array(
		array('title', 'smf_snowflakes' . $type),
		'',
		$txt['snowflakes_main'],
		'',
			array('check', 'snowflakesEnable', 1),
			array('check', 'snowflakesMobileDisable', 1),
			array('check', 'snowflakesTabletDisable', 1),
			array('check', 'snowflakesHideTranslate', 1),
			array('check', 'snowflakesNavMenu'),
			array('check', 'snowflakesSearch'),
			array('check', 'snowflakesUserMenu'),
			array('permissions', 'snowflakes_no_view'),
			array('permissions', 'snowflakes_profile_view'),
			array('select', 'snowflakesType', array($scripts[0], $scripts[1], $scripts[2], $scripts[3], $scripts[4], $scripts[5], $scripts[6], $scripts[7], $scripts[8], $scripts[99]), 'onchange' => 'this.form.submit();return false;'),
			array('check', 'snowflakesAppend'),
			array('check', 'snowflakesAdminArea'),
		'',
		$txt['snowflakes_general'],
		'',
			array('check', 'snowflakesDefaults', 1),
			array('check', 'snowflakesIgnoreRange', 1),
			array('int', 'snowflakesDisableRange', 'step' => 1, 'min' => 0, 'max' => 999),
			array('text', 'snowflakesDisableIds', $snowflakesDisableIds),
			array('text', 'snowflakesDisableClasses', $snowflakesDisableClasses),
			array('text', 'snowflakesActions', $snowflakesActions),
		'',
	);

	switch ($snowflakesSettings['snowflakesType'])
	{
		case 0:
			$config_vars[] = str_replace('&*@#', $scripts[$type], $txt['snowflakes_specific']);
			$config_vars[] = '';
			$config_vars[] = array(
				'int', 'snowflakesMinSize0', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMaxSize0', 4
			);
			$config_vars[] = array(
				'text', 'snowflakesFlakeColor0', 7
			);
			$config_vars[] = array(
				'int', 'snowflakesNewOn0', 4
			);
			break;
		case 1:
			$config_vars[] = str_replace('&*@#', $scripts[$snowflakesSettings['snowflakesType']], $txt['snowflakes_specific']);
			$config_vars[] = '';
			$config_vars[] = array(
				'int', 'snowflakesMinSize1', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMaxSize1', 4
			);
			$config_vars[] = array(
				'text', 'snowflakesFlakeColor1', 7
			);
			$config_vars[] = array(
				'int', 'snowflakesInterval1', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesNewOn1', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMinSpeed1', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMaxSpeed1', 4
			);
			$config_vars[] = array(
				'text', 'snowflakesFlakeCode1', 4
			);
			break;
		case 2:
			$config_vars[] = str_replace('&*@#', $scripts[$snowflakesSettings['snowflakesType']], $txt['snowflakes_specific']);
			$config_vars[] = '';
			$config_vars[] = array(
				'int', 'snowflakesMinSize2', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMaxSize2', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMinSpeed2', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMaxSpeed2', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesNewOn2', 4
			);
			$config_vars[] = array(
				'text', 'snowflakesCollection2', 15
			);
			break;
		case 3:
			$config_vars[] = str_replace('&*@#', $scripts[$snowflakesSettings['snowflakesType']], $txt['snowflakes_specific']);
			$config_vars[] = '';
			$config_vars[] = array(
				'check', 'snowflakesTwinkle3', 1
			);
			$config_vars[] = array(
				'check', 'snowflakesFollow3', 1
			);
			$config_vars[] = array(
				'check', 'snowflakesMelt3', 1
			);
			$config_vars[] = array(
				'check', 'snowflakesStick3', 1
			);
			$config_vars[] = array(
				'check', 'snowflakesMobile3', 1
			);
			$config_vars[] = array(
				'text', 'snowflakesFlakeColor3', 7
			);
			$config_vars[] = array(
				'int', 'snowflakesNewOn3', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMinSpeed3', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMaxSpeed3', 4
			);
			$config_vars[] = array(
				'text', 'snowflakesFlakeCode3', 4
			);
			break;
		case 4:
			$config_vars[] = str_replace('&*@#', $scripts[$snowflakesSettings['snowflakesType']], $txt['snowflakes_specific']);
			$config_vars[] = '';
			$config_vars[] = array(
				'int', 'snowflakesMinSize4', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMaxSize4', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesNewOn4', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesInterval4', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMinTime4', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMaxTime4', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesExplodeMinTime4', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesExplodeMaxTime4', 4
			);
			$config_vars[] = array(
				'text', 'snowflakesFlakeCode4', 4
			);
			break;
		case 5:
			$config_vars[] = str_replace('&*@#', $scripts[$snowflakesSettings['snowflakesType']], $txt['snowflakes_specific']);
			$config_vars[] = '';
			$config_vars[] = array(
				'int', 'snowflakesNewOn5', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesInterval5', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesMaxSpeed5', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesFlicker5', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesWidth5', 4
			);
			$config_vars[] = array(
				'int', 'snowflakesHeight5', 4
			);
			break;
		case 6:
			$config_vars[] = str_replace('&*@#', $scripts[$snowflakesSettings['snowflakesType']], $txt['snowflakes_specific']);
			$config_vars[] = '';
			if (empty($snowflakesSettings['snowflakesAuto6']))
				$config_vars[] = array(
					'message', 'snowflakesNote1_6'
				);
			$config_vars[] = array(
				'check', 'snowflakesAuto6', 4
			);
			$config_vars[] = array(
				'float', 'snowflakesOffset6', 6
			);
			$config_vars[] = array(
				'float', 'snowflakesContainerHeight6', 6
			);
			break;
		case 7:
			$config_vars[] = str_replace('&*@#', $scripts[$snowflakesSettings['snowflakesType']], $txt['snowflakes_specific']);
			$config_vars[] = '';
			if (empty($snowflakesSettings['snowflakesAuto7']))
				$config_vars[] = array(
					'message', 'snowflakesNote1_7'
				);
			$config_vars[] = array(
				'check', 'snowflakesAuto7', 4
			);
			$config_vars[] = array(
				'float', 'snowflakesOffset7', 6
			);
			$config_vars[] = array(
				'float', 'snowflakesContainerHeight7', 6
			);
			break;
		case 8:
			$config_vars[] = str_replace('&*@#', $scripts[$snowflakesSettings['snowflakesType']], $txt['snowflakes_specific']);
			$config_vars[] = '';
			if (empty($snowflakesSettings['snowflakesAuto8']))
				$config_vars[] = array(
					'message', 'snowflakesNote1_8'
				);
			$config_vars[] = array(
				'check', 'snowflakesAuto8', 4
			);
			$config_vars[] = array(
				'float', 'snowflakesOffset8', 6
			);
			$config_vars[] = array(
				'float', 'snowflakesContainerHeight8', 6
			);
			break;
		case 99:
			$config_vars[] = str_replace('&*@#', $scripts[$snowflakesSettings['snowflakesType']], $txt['snowflakes_specific']);
			$config_vars[] = '';
			foreach ($scripts as $id => $script)
			{
				if ($id == 99)
					continue;
				$config_vars[] = array(
					'check', 'smf_snowflakes' . $id, 1
				);
			}
			$config_vars[] = array(
				'check', 'snowflakesAll', 'onclick' => 'snowflakes_all(this);'
			);
			break;
		default:
			$config_vars[] = $txt['snowflakes_no_opts'];
	}

	if (in_array($type, $snowImages))
	{
		$config_vars[] = array(
				'select', 'snowflakesImage' . $type, $context['snowflakes_images'], 'onchange' => 'this.form.submit();return false;'
			);
		$config_vars[] = '<input type="file" name="file" id="file" accept="image/*" />';


	}

	if ($return_config)
		return $config_vars;

	$context['post_url'] = $scripturl . '?action=admin;area=modsettings;save;sa=snowflakes';
	$context['page_title'] = $txt['smf_snowflakes_title'] . ' - ' . $txt['settings'];
	$context['sub_template'] = 'show_settings';
	$context['settings_insert_below'] = !empty($context['settings_insert_below']) ? $context['settings_insert_below'] : '';
	$context['settings_insert_below'] .= '	<script type="text/javascript"><!-- // --><![CDATA[
	function smfsnowflakesOnload() {
		var imageId = document.getElementById("snowflakesImage" + ' . $type . ');
		if(imageId != null) {
			imageId.options[imageId.selectedIndex].text = "' . (!empty($snowflakesSettings['snowflakesImage' . $type]) ? $snowflakesSettings['snowflakesImage' . $type] : $imageTypeDefault[$type]) . '";
			var formsCollection = document.getElementById("admin_form_wrapper");
			formsCollection.enctype = "multipart/form-data";
		}

		var memberPerms = document.getElementsByName("snowflakes_profile_view[-1]");
		if (memberPerms)
		{
			memberPerms[0].parentNode.style.display = "none";
			memberPerms[0].value = 0;
		}
	}
	if (window.addEventListener)
		window.addEventListener("load", smfsnowflakesOnload, false);
	else if (window.attachEvent)
		window.attachEvent("onload", smfsnowflakesOnload);
	else
		window.onload = smfsnowflakesOnload;
	function snowflakes_all(toggle) {

		if (toggle.checked == true)
		{
			for (i=0; i<' . (count($scripts) - 1) . '; i++)
				document.getElementById("smf_snowflakes" + i).checked = true;
		}
		else
		{
			for (i=0; i<' . (count($scripts) - 1) . '; i++)
				document.getElementById("smf_snowflakes" + i).checked = false;
		}
	}
	// ]]></script>';

	if (empty($config_vars))
	{
		$context['settings_save_dont_show'] = true;
		$context['settings_message'] = '<div style="text-align: center">' . $txt['modification_no_misc_settings'] . '</div>';

		return smf_snowflakes_prepareDBSettingContext($config_vars);
	}

	if (isset($_GET['save']))
	{
		checkSession();

		// ensure inputs are within limits
		if (!empty($_POST))
		{
			if (!empty($_POST['snowflakesDefaults']))
			{
				$_POST = smf_snowflakes_defaults($snowflakesSettings);
				$_POST['snowflakesDefaults'] = 0;
			}

			$_POST['snowflakesFiles'] = 0;
			foreach($_POST as $key => $postVal)
			{
				$post = intval($postVal);
				if (!empty($_FILES['file']) && $key !== 'snowflakesImage' . $type)
				{
					$file = !empty($_FILES['file']['name']) ? $_FILES['file']['name'] : '';
					foreach (array('.gif', '.png', '.jpg', '.bmp', '.jpeg') as $fileType)
					{
						$i = strlen($fileType);
						if ((strlen($file) > $i) && substr(strtolower(trim($file)), ($i*-1), $i) === $fileType)
						{
							$val = $_FILES["file"];
							$newFile = $smcFunc['strtolower']($val["name"]);
							if (!file_exists($settings['default_theme_dir'] . '/images/' . $imagePath[$type] . '/' . $newFile))
							{
								move_uploaded_file($val["tmp_name"], $settings['default_theme_dir'] . '/images/' . $imagePath[$type] . '/' . $newFile);
								smf_snowflakes_imageResize($settings['default_theme_dir'] . '/images/' . $imagePath[$type] . '/' . $newFile, $settings['default_theme_dir'] . '/images/' . $imagePath[$type] . '/' . $newFile, 30, 30, 1);
								$_POST['snowflakesImage' . (string)$type] = $newFile;
							}
						}
					}

					continue;
				}

				switch ($key)
				{
					case 'snowflakesFlakeColor' . $type:
						$_POST['snowflakesFlakeColor' . (string)$type] = !empty($_POST['snowflakesFlakeColor' . (string)$type]) ? smf_snowflakes_colorFilter($_POST['snowflakesFlakeColor' . (string)$type]) : '#0099FF';
						break;
					case 'snowflakesFlakeCode' . $type:
						// single character or entity ... multiple char becomes first char & malformed entity becomes &bull;
						$_POST['snowflakesFlakeCode' . (string)$type] = !empty($_POST['snowflakesFlakeCode' . (string)$type]) ? trim(htmlentities($_POST['snowflakesFlakeCode' . (string)$type], ENT_QUOTES, 'UTF-8', FALSE)) : '&bull;';
						if (strlen($_POST['snowflakesFlakeCode' . (string)$type]) > 1)
						{
							$entities = get_html_translation_table(HTML_ENTITIES, ENT_QUOTES | ENT_HTML5);
							$firstEntity = explode('&', $_POST['snowflakesFlakeCode' . (string)$type]);
							$entity = !empty($firstEntity[0]) ? substr($firstEntity[0], 0, 1) : '&bull;';
							if ((count($firstEntity) > 1) && in_array('&' . $firstEntity[1], $entities))
								$_POST['snowflakesFlakeCode' . $type] = '&' . $firstEntity[1];
							else
								$_POST['snowflakesFlakeCode' . (string)$type] = $entity;
						}
						break;
					case 'snowflakes_no_view':
						break;
					case 'snowflakes_profile_view':
						break;
					case $context['session_var']:
						break;
					case 'snowflakesCollection' . $type:
						break;
					case 'snowflakesImage' . $type:
						if ($txt['smf_snowflakes_select'] === $context['snowflakes_images'][$post])
							$_POST[$key] = $snowflakesSettings['snowflakesImage' . $type];
						else
							$_POST[$key] = $context['snowflakes_images'][$post];
						break;
					case 'width5':
						$_POST[$key] = abs($post) > 500 ? 500 : abs($post);
						break;
					case 'height5':
						$_POST[$key] = abs($post) > 500 ? 500 : abs($post);
						break;
					case 'snowflakesTimeMin4':
						$_POST[$key] = abs($post) < $snowflakesSettings['snowflakesTimeMax4'] && abs($post) > 49 ? abs($post) : ($snowflakesSettings['snowflakesTimeMax4'] > 99 ? $snowflakesSettings['snowflakesTimeMax4'] - 50 : 50);
						break;
					case 'snowFlakesTimeMax4':
						$_POST[$key] = abs($post) > $snowflakesSettings['snowflakesTimeMin4'] && abs($post) > 99 ? abs($post) : ($snowflakesSettings['snowflakesTimeMin4'] > 49 ? $snowflakesSettings['snowflakesTimeMin4'] + 50 : 100);
						break;
					case 'snowflakesExplodeTimeMin4':
						$_POST[$key] = abs($post) >= $snowflakesSettings['snowflakesTimeMin4'] && abs($post) < $snowflakesSettings['snowflakesExplodeTimeMax4'] ? abs($post) : ($snowflakesSettings['snowflakesTimeMin4'] > 49 ? $snowflakesSettings['snowflakesTimeMin4'] : 50);
						break;
					case 'snowflakesExplodeTimeMax4':
						$_POST[$key] = abs($post) <= $snowflakesSettings['snowflakesTimeMax4'] && abs($post) > $snowflakesSettings['snowflakesExplodeTimeMin4'] ? abs($post) : ($snowflakesSettings['snowflakesTimeMax4'] > 99 ? $snowflakesSettings['snowflakesTimeMax4'] : 100);
						break;
					case 'snowflakesAll':
						$_POST[$key] = 0;
						break;
					case 'snowflakesType':
						$optType = !empty($_POST[$key]) ? $_POST[$key] : 0;
						$countVals = 0;
						if ($optType > count($scripts) - 2)
						{
							for ($i=0; $i < (count($scripts) - 2); $i++)
								$countVals += !empty($_POST['smf_snowflakes' . $i]) ? (int)$_POST['smf_snowflakes' . $i] : 0;

							if (empty($countVals))
								$_POST['smf_snowflakes0'] = 1;
						}
						break;
					case 'snowflakesOffset6':
						$post = round((float)$post, 2);
						$_POST[$key] = $post <= -11 ? -10.99 : ($post >= 11 ? 10.99 : $post);
						break;
					case 'snowflakesContainerHeight6':
						$post = round((float)$post, 2);
						$_POST[$key] = $post < 1 ? 4.5 : ($post >= 20 ? 19.99 : $post);
						break;
					case 'snowflakesOffset7':
						$post = round((float)$post, 2);
						$_POST[$key] = $post <= -11 ? -10.99 : ($post >= 11 ? 10.99 : $post);
						break;
					case 'snowflakesContainerHeight7':
						$post = round((float)$post, 2);
						$_POST[$key] = $post < 1 ? 4.5 : ($post >= 20 ? 19.99 : $post);
						break;
					case 'snowflakesOffset8':
						$post = round((float)$post, 2);
						$_POST[$key] = $post <= -11 ? -10.99 : ($post >= 11 ? 10.99 : $post);
						break;
					case 'snowflakesContainerHeight8':
						$post = round((float)$post, 2);
						$_POST[$key] = $post < 1 ? 4.5 : ($post >= 20 ? 19.99 : $post);
						break;
					case 'snowflakesActions':
						$post = preg_replace('/[^A-Za-z0-9,_~|]/', '', $post);
						break;
					case 'snowflakesDisableRange':
						$post = intval($post);
						$post = abs($post);
						$post = $post > -1 && $post < 1000 ? $post : ($post > 999 ? 999 : 0);
						$_POST[$key] = $post;
						break;
					case 'snowflakesIgnoreRange':
						$post = intval($post) == 1 ? 1 : 0;
						$_POST[$key] = $post;
						break;
					case 'snowflakesDisableIds':
						$post = preg_replace('/[^A-Za-z0-9,_]/', '', $post);
						break;
					case 'snowflakesDisableClasses':
						$post = preg_replace('/[^A-Za-z0-9,_]/', '', $post);
						break;
					default:
						$_POST[$key] = abs($post);
				}
			}
		}

		$save_vars = $config_vars;
		smf_snowflakes_saveDBSettings($save_vars);
		$_SESSION['adm-save'] = true;
		redirectexit($scripturl . '?action=admin;area=modsettings;sa=snowflakes');
	}

	smf_snowflakes_prepareDBSettingContext($config_vars);
}

function smf_snowflakes_buffer($buffer)
{
	global $settings, $context, $user_info, $modSettings, $smcFunc, $boardurl;

	$adminArea = !empty($context['admin_area']) ? $context['admin_area'] : '';
	if ($context['current_action'] == 'admin' && $adminArea == 'modsettings' && $context['current_subaction'] != 'snowflakes')
		return $buffer;
	elseif ($context['current_action'] == 'admin' && $adminArea != 'modsettings')
		return $buffer;

	@ini_set('memory_limit', '128M');
	$request = $smcFunc['db_query']('', '
		SELECT disable
		FROM {db_prefix}snowflakes_members
		WHERE user_id = {int:member}',
		array(
			'member' => $user_info['id'],
		)
	);
	list ($disabled) = $smcFunc['db_fetch_row']($request);
	$smcFunc['db_free_result'];

	if (!empty($disabled))
		return $buffer;

	$scripts = smf_snowflakes_scripts();
	list($imageTypeDefault, $imagePath, $imageTypeDefault[99], $imagePath[99]) = array(array(), array(), '', '');
	$snowflakesSettings = smf_snowflakes_getSettings();
	$enabled = !empty($snowflakesSettings['snowflakesEnable']) ? (int)$snowflakesSettings['snowflakesEnable'] : (isset($modSettings['snowflakesTypeOverride']) ? 1 : 0);
	$actions = !empty($snowflakesSettings['snowflakesActions']) ? explode(',', preg_replace('/[^A-Za-z0-9,_~|]/', '', $snowflakesSettings['snowflakesActions'])) : array();
	$snowIds = !empty($snowflakesSettings['snowflakesDisableIds']) ? explode(',', preg_replace('/[^A-Za-z0-9,_]/', '', $snowflakesSettings['snowflakesDisableIds'])) : array();
	$snowClasses = !empty($snowflakesSettings['snowflakesDisableClasses']) ? explode(',', preg_replace('/[^A-Za-z0-9,_]/', '', $snowflakesSettings['snowflakesDisableClasses'])) : array();
	$snowRange = !empty($snowflakesSettings['snowflakesDisableRange']) ? intval($snowflakesSettings['snowflakesDisableRange']) : 0;
	$noContextAction = isset($_REQUEST['action']) && is_string($_REQUEST['action']) ? preg_replace('/[^A-Za-z0-9,_]/', '', $_REQUEST['action']) : '';
	$noContextSubAction = isset($_REQUEST['sa']) && is_string($_REQUEST['sa']) ? preg_replace('/[^A-Za-z0-9,_]/', '', $_REQUEST['sa']) : '';
	foreach ($actions as $action)
	{
		list($actionX, $subactionX, $checkOrSubs, $checkAndSubs) = array('', '', explode('|', $action), explode('~', $action));
		if (count($checkOrSubs) > 1)
		{
			$subactionX = preg_replace('/[^A-Za-z0-9,_]/', '', $checkOrSubs[1]);
			$actionX = preg_replace('/[^A-Za-z0-9,_]/', '', $checkOrSubs[0]);
			$enabled = strcasecmp($context['current_subaction'], $subactionX) == 0 || strcasecmp($context['current_action'], $actionX) == 0 ? 0 : $enabled;
			$enabled = strcasecmp($noContextSubAction, $subactionX) == 0 || strcasecmp($noContextAction, $actionX) == 0 ? 0 : $enabled;
		}
		elseif (count($checkAndSubs) > 1)
		{
			$subactionY = preg_replace('/[^A-Za-z0-9,_]/', '', $checkAndSubs[1]);
			$actionX = preg_replace('/[^A-Za-z0-9,_]/', '', $checkAndSubs[0]);
			$enabled = strcasecmp($context['current_subaction'], $subactionY) == 0 && strcasecmp($context['current_action'], $actionX) == 0 ? 0 : $enabled;
			$enabled = strcasecmp($noContextSubAction, $subactionY) == 0 && strcasecmp($noContextAction, $actionX) == 0 ? 0 : $enabled;
		}
		else
		{
			$actionX = preg_replace('/[^A-Za-z0-9,_]/', '', $action);
			$enabled = strcasecmp($context['current_action'], $actionX) == 0 ? 0 : $enabled;
			$enabled = strcasecmp($noContextAction, $actionX) == 0 ? 0 : $enabled;
		}
	}

	if (!$enabled || $context['current_action'] === 'helpadmin')
		return $buffer;
	elseif (!$user_info['is_admin'] && allowedTo('snowflakes_no_view'))
		return $buffer;

	$type = isset($modSettings['snowflakesTypeOverride']) ? (int)$modSettings['snowflakesTypeOverride'] : (!empty($snowflakesSettings['snowflakesType']) ? (int)$snowflakesSettings['snowflakesType'] : 0);

	// random script?
	if ($type > (count($scripts) - 2) || $type == 99)
	{
		$random = array();
		for ($i=0; $i < (count($scripts) - 1); $i++)
			if (!empty($snowflakesSettings['smf_snowflakes' . $i]))
				$random[] = $i;
		$random = !empty($random) ? $random : array(0);
		shuffle($random);
		$type = $random[0];
	}

	// ignore Y coordinate for "Xmas Lights"
	$snowRangeIgnore = !empty($snowflakesSettings['snowflakesIgnoreRange']) ? intval($snowflakesSettings['snowflakesIgnoreRange']) : 0;
	$snowRangeIgnore = !empty($snowRangeIgnore) && $type == 6 ? 1 : 0;

	for ($a=0;$a<9;$a++)
	{
		$imageTypeDefault[$a] = $a == 8 ? 'wired_lights1.gif' : 'snowflake.png';
		$imagePath[$a] = $a == 8 ? 'wired_lights' : 'snowflakes';
	}
	$snowflakesSettings['snowflakesMobileDisable'] = !empty($snowflakesSettings['snowflakesMobileDisable']) ? floatval($snowflakesSettings['snowflakesMobileDisable']) : 0;
	$snowflakesSettings['snowflakesTabletDisable'] = !empty($snowflakesSettings['snowflakesTabletDisable']) ? floatval($snowflakesSettings['snowflakesTabletDisable']) : 0;
	if(!empty($snowflakesSettings['snowflakesMobileDisable']) && $snowflakesSettings['isMobile'])
		return $buffer;

	if(!empty($snowflakesSettings['snowflakesTabletDisable']) && $snowflakesSettings['isTablet'])
		return $buffer;

	if ($type < 0 || $type > (count($scripts) - 2))
		return $buffer;

	$snowflake_settings = array(
		'minSize' => !empty($snowflakesSettings['snowflakesMinSize' . $type]) ? (int)$snowflakesSettings['snowflakesMinSize' . $type] : 5,
		'maxSize' => !empty($snowflakesSettings['snowflakesMaxSize' . $type]) ? (int)$snowflakesSettings['snowflakesMaxSize' . $type] : 50,
		'newOn' => !empty($snowflakesSettings['snowflakesNewOn' . $type]) ? (int)$snowflakesSettings['snowflakesNewOn' . $type] : 1000,
		'minSpeed' => !empty($snowflakesSettings['snowflakesMinSpeed' . $type]) ? (int)$snowflakesSettings['snowflakesMinSpeed' . $type] : 1,
		'maxSpeed' => !empty($snowflakesSettings['snowflakesMaxSpeed' . $type]) ? (int)$snowflakesSettings['snowflakesMaxSpeed' . $type] : 5,
		'flakeColor' => !empty($snowflakesSettings['snowflakesFlakeColor' . $type]) ? $snowflakesSettings['snowflakesFlakeColor' . $type] : '#0099FF',
		'flakeCode' => !empty($snowflakesSettings['snowflakesFlakeCode' . $type]) ? $snowflakesSettings['snowflakesFlakeCode' . $type] : '&bull;',
		'interval' => !empty($snowflakesSettings['snowflakesInterval' . $type]) ? (int)$snowflakesSettings['snowflakesInterval' . $type] : 30,
		'image' => (!empty($snowflakesSettings['snowflakesImage' . $type]) ? $settings['default_theme_url'] . '/images/' . $imagePath[$type] . '/' . $snowflakesSettings['snowflakesImage' . $type] : $settings['default_theme_url'] . '/images/' . $imagePath[$type] . '/' . $imageTypeDefault[$type]),
		'collection' => !empty($snowflakesSettings['snowflakesCollection' . $type]) ? $snowflakesSettings['snowflakesCollection' . $type] : '',
		'followMouse' => !empty($snowflakesSettings['snowflakesFollow' . $type]) ? 'true' : 'false',
		'melt' => !empty($snowflakesSettings['snowflakesMelt' . $type]) ? 'true' : 'false',
		'snowStick' => !empty($snowflakesSettings['snowflakesStick' . $type]) ? 'true' : 'false',
		'excludeMobile' => !empty($snowflakesSettings['snowflakesMobile' . $type]) ? 'true' : 'false',
		'flicker' => !empty($snowflakesSettings['snowflakesFlicker' . $type]) ? (int)$snowflakesSettings['snowflakesFlicker' . $type] : 0,
		'twinkle' => !empty($snowflakesSettings['snowflakesTwinkle' . $type]) ? (int)$snowflakesSettings['snowflakesTwinkle' . $type] : 0,
		'width' => !empty($snowflakesSettings['snowflakesWidth' . $type]) ? (int)$snowflakesSettings['snowflakesWidth' . $type] : 0,
		'height' => !empty($snowflakesSettings['snowflakesHeight' . $type]) ? (int)$snowflakesSettings['snowflakesHeight' . $type] : 0,
		'minTime' => !empty($snowflakesSettings['snowflakesMinTime' . $type]) ? (int)$snowflakesSettings['snowflakesMinTime' . $type] : 50,
		'maxTime' => !empty($snowflakesSettings['snowflakesMaxTime' . $type]) ? (int)$snowflakesSettings['snowflakesMaxTime' . $type] : 100,
		'explodeMin' => !empty($snowflakesSettings['snowflakesExplodeMinTime' . $type]) ? (int)$snowflakesSettings['snowflakesExplodeMinTime' . $type] : 50,
		'explodeMax' => !empty($snowflakesSettings['snowflakesExplodeMaxTime' . $type]) ? (int)$snowflakesSettings['snowflakesExplodeMaxTime' . $type] : 80,
		'lights' => !empty($snowflakesSettings['snowflakesLights' . $type]) ? (int)$snowflakesSettings['snowflakesLights' . $type] : 0,
		'topOffset' => !empty($snowflakesSettings['snowflakesOffset' . $type]) ? (int)$snowflakesSettings['snowflakesOffset' . $type] : 0,
		'containerHeight' => !empty($snowflakesSettings['snowflakesContainerHeight' . $type]) ? abs((float)$snowflakesSettings['snowflakesContainerHeight' . $type]) : 4.5,
	);

	$snowflake_settings['snowflakesVersion'] = !empty($snowflake_settings['snowflakesVersion']) ? $snowflake_settings['snowflakesVersion'] : 'v' . rand(1, 99) . '.0';
	$snowflake_settings['files'] = !empty($snowflakesSettings['snowflakesFiles']) ? (int)$snowflakesSettings['snowflakesFiles'] : 0;
	$snowflake_settings['append'] = !empty($snowflakesSettings['snowflakesAppend']) ? (int)$snowflakesSettings['snowflakesAppend'] : 0;
	$snowflake_settings['admin'] = !empty($snowflakesSettings['snowflakesAdminArea']) ? (int)$snowflakesSettings['snowflakesAdminArea'] : 0;
	$snowflake_settings['translate'] = !empty($snowflakesSettings['snowflakesHideTranslate']) ? (int)$snowflakesSettings['snowflakesHideTranslate'] : 0;
	$currentRequest = isset($_REQUEST['action']) && is_string($_REQUEST['action']) ? filter_var($_REQUEST['action'], FILTER_SANITIZE_STRING) : '';
	$currentAction = isset($context['current_action']) ? $context['current_action'] : $currentRequest;

	if (empty($snowflake_settings['admin']) && $currentAction === 'admin')
		return $buffer;

	$css = '
	<link rel="stylesheet" href="' . $settings['default_theme_url'] . '/css/smf_snowflakes.css" />
	<style>
		.xlsf-fragment {
			position: absolute;
			background: transparent url(' . $boardurl . '/lights/image/bulbs-50x50-fragments.png) no-repeat 0px 0px;
			width: 50px;
			height: 50px;
		}' . (!empty($snowflake_settings['translate']) ? '
		iframe.goog-te-banner-frame.skiptranslate {
			display: none !important;
			height: 0em !important;
			max-height: 0em !important;
			width: 0em !important;
			max-width: 0em !important;
			visibility: hidden !important;
		}
		body {
			top: 0em !important;
		}' : '') . '
	</style>';

	// create JavaScript for ids and classes for which we want to disable lights
	list($delId, $delClass, $snowDelIds, $snowDelClasses, $snowDelIdNegs, $snowDelClassNegs, $snowDelRange, $snowJsFunctions) = array(0, 0, '', '', '', '', '', '');
	$lightsPanel = array('nil', 'nil', 'nil', 'nil', 'nil', 'nil', 'Lights', 'Strand', 'LightsPanel');
	if (!empty($snowRange) && empty($snowRangeIgnore)) {
		$snowDelRange = '
		let bodyZ = document.body;
		let snowDelRange = ' . $snowRange . ';
		bodyZ.onmousemove=function(e) {
			let snowBodyY = e.clientY;
			document.body.onmouseover = function(){
				if (snowBodyY > 1 && snowBodyY <= snowDelRange) {
					document.getElementById("snowflakes%s").style.display="none";
				}
				else {
					document.getElementById("snowflakes%s").style.display="flex";
				}
			};
			document.head.onmouseover = function(){
				console.log("head");
				if (snowBodyY > 1 && snowBodyY <= snowDelRange) {
					document.getElementById("snowflakes%s").style.display="none";
				}
				else {
					document.getElementById("snowflakes%s").style.display="flex";
				}
			};
			document.body.onmouseout = function(){
				document.head.onmouseout = function() {
					document.getElementById("snowflakes%s").style.display="flex";
				};
			};
		};';
	}
	foreach ($snowIds as $snowId)
	{
		$snowDelIds .= '
		var snowDelId' . $delId . ' = document.getElementById("' . $snowId. '");';

		$snowDelIdNegs .= '
		if (snowDelId' . $delId . ')
		{
			snowDelId' . $delId . '.style.zIndex = "9999";
			snowDelId' . $delId . '.style.position = "relative";
			snowDelId' . $delId . '.onmouseover = function(){document.getElementById("snowflakes%s").style.display="none";};
			snowDelId' . $delId . '.onmouseout = function(){document.getElementById("snowflakes%s").style.display="flex";};
		}';

		$delId++;
	}
	foreach ($snowClasses as $snowClass)
	{
		$snowDelClasses .= '
		var snowDelClass' . $delClass . ' = document.getElementsByClassName("' . $snowClass. '").length > 0 ? document.getElementsByClassName("' . $snowClass. '")[0] : false;';

		$snowDelClassNegs .= '
		if (snowDelClass' . $delClass . ')
		{
			snowDelClass' . $delClass . '.style.zIndex = "9999";
			snowDelClass' . $delClass . '.style.position = "relative";
			snowDelClass' . $delClass . '.onmouseover = function(){document.getElementById("snowflakes%s").style.display="none";};
			snowDelClass' . $delClass . '.onmouseout = function(){document.getElementById("snowflakes%s").style.display="flex";};
		}';

		$delClass++;
	}

	$snowJsFunctions = '	<script type="text/javascript" id="snowScriptId">
		function snowDelIds() {
			' . $snowDelIds . $snowDelIdNegs . '
			return false;
		}
		function snowDelClasses() {
			' . $snowDelClasses . $snowDelClassNegs . '
			return false;
		}
		function snowDelRange() {
			' . $snowDelRange . '
			return false;
		}
	</script>';
	$snowJsFunctions = str_replace('%s', $lightsPanel[$type], $snowJsFunctions);

	$snowType = array(
		0 => '
	<script type="text/javascript">
		var snowflake_settings_minSize = ' . $snowflake_settings['minSize'] . ',
		snowflake_settings_maxSize = ' . $snowflake_settings['maxSize'] . ',
		snowflake_settings_newOn = ' . $snowflake_settings['newOn'] . ',
		snowflake_settings_flakeColor = "' . $snowflake_settings['flakeColor'] . '";
	</script>',
		1 => '
	<script type="text/javascript"><!-- // --><![CDATA[
		var snowflake_settings_flakeColor = "' . $snowflake_settings['flakeColor'] . '",
		snowflake_settings_flakeCode = "' . ($snowflake_settings['flakeCode'] === '&quot;' ? '\\' : '') . htmlspecialchars_decode($snowflake_settings['flakeCode'], ENT_QUOTES) . '",
		snowflake_settings_minSpeed = ' . $snowflake_settings['minSpeed'] . ',
		snowflake_settings_maxSpeed = ' . $snowflake_settings['maxSpeed'] . ',
		snowflake_settings_maxSize = ' . $snowflake_settings['maxSize'] . ',
		snowflake_settings_minSize = ' . $snowflake_settings['minSize'] . ',
		snowflake_settings_interval  =  "' . $snowflake_settings['interval'] . '",
		snowflake_settings_newOn = ' . $snowflake_settings['newOn'] . ';
	// ]]></script>',
		2 => '
	<script type="text/javascript">
		var snowflake_settings_image = "' . $snowflake_settings['image'] . '",
		snowflake_settings_minSize = ' . $snowflake_settings['minSize'] . ',
		snowflake_settings_maxSize = ' . $snowflake_settings['maxSize'] . ',
		snowflake_settings_minSpeed = ' . $snowflake_settings['minSpeed'] . ',
		snowflake_settings_maxSpeed = ' . $snowflake_settings['maxSpeed'] . ',
		snowflake_settings_flakeColor = "' . $snowflake_settings['flakeColor'] . '",
		snowflake_settings_collection = ' . (!empty($snowflake_settings['collection']) ? '"' . $snowflake_settings['collection'] . '"' : '""') .  ',
		snowflake_settings_newOn = ' . $snowflake_settings['newOn'] . ';
	</script>',
		3 => $type == 3 ? '
	<script type="text/javascript"><!-- // --><![CDATA[
		var snowflake_settings_flakeColor = "' . $snowflake_settings['flakeColor'] . '",
		snowflake_settings_newOn = ' . $snowflake_settings['newOn'] . ',
		snowflake_settings_twinkle = ' . ($snowflake_settings['twinkle'] > 0 ? 'true' : 'false') . ',
		$snowflake_settings_flakeCode = "' . ($snowflake_settings['flakeCode'] === '&quot;' ? '\\' : '') . htmlspecialchars_decode($snowflake_settings['flakeCode'], ENT_QUOTES) . '",
		snowflake_settings_collection = ' . (!empty($snowflake_settings['collection']) ? '"' . $snowflake_settings['collection'] . '"' : '""') .  ',
		snowflake_settings_followMouse = ' . $snowflake_settings['followMouse'] . ',
		snowflake_settings_melt = ' . $snowflake_settings['melt'] . ',
		snowflake_settings_snowStick = ' . $snowflake_settings['snowStick'] . ',
		snowflake_settings_excludeMobile = ' . $snowflake_settings['excludeMobile'] . ',
		snowflake_settings_newOn = ' . $snowflake_settings['newOn'] . ',
		snowflake_settings_minSpeed = ' . $snowflake_settings['minSpeed'] . ',
		snowflake_settings_maxSpeed = ' . $snowflake_settings['maxSpeed'] . ',
		snowflake_settings_width = ' . (!empty($snowflake_settings['width']) ? $snowflake_settings['width'] : '8') . ',
		snowflake_settings_height = ' . (!empty($snowflake_settings['height']) ? $snowflake_settings['height'] : '8') . ';
	// ]]></script>' : '',
		4 => '
	<script type="text/javascript"><!-- // --><![CDATA[
		var snowflake_settings_newOn = ' . $snowflake_settings['newOn'] . ',
		snowflake_settings_interval = ' . $snowflake_settings['interval'] . ',
		snowflake_settings_flakeCode = "<div>' . ($snowflake_settings['flakeCode'] === '&quot;' ? '\\' : '') . htmlspecialchars_decode($snowflake_settings['flakeCode'], ENT_QUOTES) . '</div>",
		snowflake_settings_maxTime = ' . $snowflake_settings['maxTime'] . ',
		snowflake_settings_minTime = ' . $snowflake_settings['minTime'] . ',
		snowflake_settings_maxSize =  ' . $snowflake_settings['maxSize'] . ',
		snowflake_settings_minSize =  ' . $snowflake_settings['minSize'] . ',
		snowflake_settings_explodeMax =  ' . $snowflake_settings['explodeMax'] . ',
		snowflake_settings_explodeMin =  ' . $snowflake_settings['explodeMin'] . ';
	// ]]></script>',
		5 => '
	<script type="text/javascript">
		var snowflake_settings_image = "' . $snowflake_settings['image'] . '",
		snowflake_settings_newOn = ' . $snowflake_settings['newOn'] . ',
		snowflake_settings_width =  ' . $snowflake_settings['width'] . ',
		snowflake_settings_height = ' . $snowflake_settings['height'] . ',
		snowflake_settings_interval = ' . $snowflake_settings['interval'] . ',
		snowflake_settings_maxSpeed = ' . $snowflake_settings['maxSpeed'] . ',
		snowflake_settings_flicker =  ' . $snowflake_settings['flicker'] . ';
	</script>',
		6 => '
	<script type="text/javascript">
		var snowflakesSettings_snowflakesSearch = ' . (!empty($snowflakesSettings['snowflakesSearch']) ? 'true' : 'false') . ',
		snowflakesSettings_snowflakesNavMenu = ' . (!empty($snowflakesSettings['snowflakesNavMenu']) ? 'true' : 'false') . ',
		snowflakesSettings_snowflakesUserMenu = ' . (!empty($snowflakesSettings['snowflakesUserMenu']) ? 'true' : 'false') . ',
		snowflakesSettings_snowflakesSearch = ' . (!empty($snowflakesSettings['snowflakesSearch']) ? 'true' : 'false') . ',
		snowflakesSettings_snowLoadLightsStyle = "height: ' . $snowflake_settings['containerHeight'] . 'em;top: ' . $snowflake_settings['topOffset'] . 'em;";
	</script>',
		7 =>'
	<script type="text/javascript">
		var snowflakesSettings_snowflakesSearch = ' . (!empty($snowflakesSettings['snowflakesSearch']) ? 'true' : 'false') . ',
		snowflakesSettings_snowflakesNavMenu = ' . (!empty($snowflakesSettings['snowflakesNavMenu']) ? 'true' : 'false') . ',
		snowflakesSettings_snowflakesUserMenu = ' . (!empty($snowflakesSettings['snowflakesUserMenu']) ? 'true' : 'false') . ',
		snowflakesSettings_snowflakesSearch = ' . (!empty($snowflakesSettings['snowflakesSearch']) ? 'true' : 'false') . ',
		snowflakesSettings_snowLoadLightsStyle = "height: ' . $snowflake_settings['containerHeight'] . 'em;top: ' . $snowflake_settings['topOffset'] . 'em;";
	</script>',
		8 => '
	<script type="text/javascript">
		var snowflake_settings_containerHeight = ' . $snowflake_settings['containerHeight'] . ',
		snowflake_settings_topOffset = ' . $snowflake_settings['topOffset'] . ',
		snowflake_settings_image = "' . $snowflake_settings['image'] . '",
		snowflakesSettings_snowflakesUserMenu = ' . (!empty($snowflakesSettings['snowflakesUserMenu']) ? 'true' : 'false') . ',
		snowflakesSettings_snowflakesSearch = ' . (!empty($snowflakesSettings['snowflakesSearch']) ? 'true' : 'false') . ',
		snowflakesSettings_snowflakesNavMenu = ' . (!empty($snowflakesSettings['snowflakesNavMenu']) ? 'true' : 'false') . ',
		snowflakesSettings_snowLoadLightsStyle = "height: ' . $snowflake_settings['containerHeight'] . 'em;top: ' . $snowflake_settings['topOffset'] . 'em;";
	</script>');

	$add = $snowJsFunctions . $snowType[$type] . $css;
	if ($type == 6)
	{
		$temp_buffer = snowflakes_str_replace('</body>', '	<script type="text/javascript">
		var snowLoadLights = document.createElement("DIV");
		snowLoadLights.id = "snowflakesLights";
		snowLoadLights.className = "notranslate";
		snowLoadLights.style = "height: ' . $snowflake_settings['containerHeight'] . 'em;top: ' . $snowflake_settings['topOffset'] . 'em;";
		snowLoadLights.innerHTML = "<!-- lights go here -->";
		document.getElementsByTagName("BODY")[0].insertBefore(snowLoadLights, document.getElementsByTagName("BODY")[0].firstChild);
	</script>
</body>', $buffer, true);
		$buffer = $temp_buffer;
	}
	if (empty($snowflake_settings['append']))
	{
		$new_buffer = snowflakes_str_replace('</head>', $add . '
	<script type="text/javascript" src="' . $settings['default_theme_url'] . '/scripts/snowflakes.ionSound.js?' . $snowflake_settings['snowflakesVersion'] . '"></script>
	<script type="text/javascript" src="' . $settings['default_theme_url'] . '/scripts/jquery.snow' . strval($type) . '.js?' . $snowflake_settings['snowflakesVersion'] . '"></script>
	<script type="text/javascript" src="' . $settings['default_theme_url'] . '/scripts/snowflakes.load.js>' . $snowflake_settings['snowflakesVersion'] . '"></script>
	<script type="text/javascript">
		if (window.addEventListener && typeof smfsnowflakesOnload' . strval($type) . ' === "function") {' . (in_array($type, array(6,7,8)) ? '
			window.addEventListener("load", snowLights' . strval($type) . ', false);' : '') . '
			window.addEventListener("load", smfsnowflakesOnload' . strval($type) . ', false);
		}
		else if (window.attachEvent && typeof smfsnowflakesOnload' . strval($type) . ' === "function") {' . (in_array($type, array(6,7,8)) ? '
			window.attachEvent("onload", snowLights' . strval($type) . ');' : '') . '
			window.attachEvent("onload", smfsnowflakesOnload' . strval($type) . ');
		}
	</script>
</head>', $buffer, false);
	}
	else
	{
		$new_buffer = snowflakes_str_replace('</head>', $add . '
</head>', $buffer, false);
		$new_buffer = snowflakes_str_replace('</body>', '	<script type="text/javascript" src="' . $settings['default_theme_url'] . '/scripts/snowflakes.ionSound.js?' . $snowflake_settings['snowflakesVersion'] . '"></script>
	<script type="text/javascript" src="' . $settings['default_theme_url'] . '/scripts/jquery.snow' . strval($type) . '.js?' . $snowflake_settings['snowflakesVersion'] . '"></script>
	<script type="text/javascript" src="' . $settings['default_theme_url'] . '/scripts/snowflakes.load.js? ' . $snowflake_settings['snowflakesVersion'] . '"></script>
	<script type="text/javascript">
		if (window.addEventListener) {' . (in_array($type, array(6,7,8)) ? '
			window.addEventListener("load", snowLights' . strval($type) . ', false);' : '') . '
			window.addEventListener("load", smfsnowflakesOnload' . strval($type) . ', false);
		}
		else if (window.attachEvent) {' . (in_array($type, array(6,7,8)) ? '
			window.attachEvent("onload", snowLights' . strval($type) . ');' : '') . '
			window.attachEvent("onload", smfsnowflakesOnload' . strval($type) . ');
		}
	</script>
</body>', $new_buffer, true);
	}

	return $new_buffer;
}

function snowflakes_str_replace($search, $replace, $string, $reverse = false)
{
    $pos = empty($reverse) ? strpos($string, $search) : strrpos($string, $search);
    if ($pos !== false)
        return substr_replace($string, $replace, $pos, strlen($search));

    return $string;
}

function smf_snowflakes_hex_filter($color)
{
	if ($color[0] == '#')
		$color = substr($color, 1);

	if (strlen($color) == 6)
		list($r, $g, $b) = array(
			$color[0].$color[1],
			$color[2].$color[3],
			$color[4].$color[5]
		);
	elseif (strlen($color) == 3)
		list($r, $g, $b) = array(
			$color[0].$color[0],
			$color[1].$color[1],
			$color[2].$color[2]
		);
	else
		return '0099FF';

	$r = hexdec($r);
	$g = hexdec($g);
	$b = hexdec($b);
	if (is_array($r) && sizeof($r) == 3)
		list($r, $g, $b) = $r;

	$r = intval($r);
	$g = intval($g);
	$b = intval($b);

	$r = dechex($r<0?0:($r>255?255:$r));
	$g = dechex($g<0?0:($g>255?255:$g));
	$b = dechex($b<0?0:($b>255?255:$b));

	$color = (strlen($r) < 2?'0':'') . $r;
	$color .= (strlen($g) < 2?'0':'') . $g;
	$color .= (strlen($b) < 2?'0':'') . $b;

	return $color;
}

function smf_snowflakes_colorFilter($color)
{
	// standard 147 HTML color names
	$colors = array(
		'aliceblue'=>'f0f8ff',
		'antiquewhite'=>'faebd7',
		'aqua'=>'00ffff',
		'aquamarine'=>'7fffd4',
		'azure'=>'f0ffff',
		'beige'=>'f5f5dc',
		'bisque'=>'ffe4c4',
		'black'=>'000000',
		'blanchedalmond '=>'ffebcd',
		'blue'=>'0000ff',
		'blueviolet'=>'8a2be2',
		'brown'=>'a52a2a',
		'burlywood'=>'deb887',
		'cadetblue'=>'5f9ea0',
		'chartreuse'=>'7fff00',
		'chocolate'=>'d2691e',
		'coral'=>'ff7f50',
		'cornflowerblue'=>'6495ed',
		'cornsilk'=>'fff8dc',
		'crimson'=>'dc143c',
		'cyan'=>'00ffff',
		'darkblue'=>'00008b',
		'darkcyan'=>'008b8b',
		'darkgoldenrod'=>'b8860b',
		'darkgray'=>'a9a9a9',
		'darkgreen'=>'006400',
		'darkgrey'=>'a9a9a9',
		'darkkhaki'=>'bdb76b',
		'darkmagenta'=>'8b008b',
		'darkolivegreen'=>'556b2f',
		'darkorange'=>'ff8c00',
		'darkorchid'=>'9932cc',
		'darkred'=>'8b0000',
		'darksalmon'=>'e9967a',
		'darkseagreen'=>'8fbc8f',
		'darkslateblue'=>'483d8b',
		'darkslategray'=>'2f4f4f',
		'darkslategrey'=>'2f4f4f',
		'darkturquoise'=>'00ced1',
		'darkviolet'=>'9400d3',
		'deeppink'=>'ff1493',
		'deepskyblue'=>'00bfff',
		'dimgray'=>'696969',
		'dimgrey'=>'696969',
		'dodgerblue'=>'1e90ff',
		'firebrick'=>'b22222',
		'floralwhite'=>'fffaf0',
		'forestgreen'=>'228b22',
		'fuchsia'=>'ff00ff',
		'gainsboro'=>'dcdcdc',
		'ghostwhite'=>'f8f8ff',
		'gold'=>'ffd700',
		'goldenrod'=>'daa520',
		'gray'=>'808080',
		'green'=>'008000',
		'greenyellow'=>'adff2f',
		'grey'=>'808080',
		'honeydew'=>'f0fff0',
		'hotpink'=>'ff69b4',
		'indianred'=>'cd5c5c',
		'indigo'=>'4b0082',
		'ivory'=>'fffff0',
		'khaki'=>'f0e68c',
		'lavender'=>'e6e6fa',
		'lavenderblush'=>'fff0f5',
		'lawngreen'=>'7cfc00',
		'lemonchiffon'=>'fffacd',
		'lightblue'=>'add8e6',
		'lightcoral'=>'f08080',
		'lightcyan'=>'e0ffff',
		'lightgoldenrodyellow'=>'fafad2',
		'lightgray'=>'d3d3d3',
		'lightgreen'=>'90ee90',
		'lightgrey'=>'d3d3d3',
		'lightpink'=>'ffb6c1',
		'lightsalmon'=>'ffa07a',
		'lightseagreen'=>'20b2aa',
		'lightskyblue'=>'87cefa',
		'lightslategray'=>'778899',
		'lightslategrey'=>'778899',
		'lightsteelblue'=>'b0c4de',
		'lightyellow'=>'ffffe0',
		'lime'=>'00ff00',
		'limegreen'=>'32cd32',
		'linen'=>'faf0e6',
		'magenta'=>'ff00ff',
		'maroon'=>'800000',
		'mediumaquamarine'=>'66cdaa',
		'mediumblue'=>'0000cd',
		'mediumorchid'=>'ba55d3',
		'mediumpurple'=>'9370d0',
		'mediumseagreen'=>'3cb371',
		'mediumslateblue'=>'7b68ee',
		'mediumspringgreen'=>'00fa9a',
		'mediumturquoise'=>'48d1cc',
		'mediumvioletred'=>'c71585',
		'midnightblue'=>'191970',
		'mintcream'=>'f5fffa',
		'mistyrose'=>'ffe4e1',
		'moccasin'=>'ffe4b5',
		'navajowhite'=>'ffdead',
		'navy'=>'000080',
		'oldlace'=>'fdf5e6',
		'olive'=>'808000',
		'olivedrab'=>'6b8e23',
		'orange'=>'ffa500',
		'orangered'=>'ff4500',
		'orchid'=>'da70d6',
		'palegoldenrod'=>'eee8aa',
		'palegreen'=>'98fb98',
		'paleturquoise'=>'afeeee',
		'palevioletred'=>'db7093',
		'papayawhip'=>'ffefd5',
		'peachpuff'=>'ffdab9',
		'peru'=>'cd853f',
		'pink'=>'ffc0cb',
		'plum'=>'dda0dd',
		'powderblue'=>'b0e0e6',
		'purple'=>'800080',
		'red'=>'ff0000',
		'rosybrown'=>'bc8f8f',
		'royalblue'=>'4169e1',
		'saddlebrown'=>'8b4513',
		'salmon'=>'fa8072',
		'sandybrown'=>'f4a460',
		'seagreen'=>'2e8b57',
		'seashell'=>'fff5ee',
		'sienna'=>'a0522d',
		'silver'=>'c0c0c0',
		'skyblue'=>'87ceeb',
		'slateblue'=>'6a5acd',
		'slategray'=>'708090',
		'slategrey'=>'708090',
		'snow'=>'fffafa',
		'springgreen'=>'00ff7f',
		'steelblue'=>'4682b4',
		'tan'=>'d2b48c',
		'teal'=>'008080',
		'thistle'=>'d8bfd8',
		'tomato'=>'ff6347',
		'turquoise'=>'40e0d0',
		'violet'=>'ee82ee',
		'wheat'=>'f5deb3',
		'white'=>'ffffff',
		'whitesmoke'=>'f5f5f5',
		'yellow'=>'ffff00',
		'yellowgreen'=>'9acd32'
	);

	$color = strtolower($color);
	if (!empty($colors[$color]))
		return ('#' . $colors[$color]);
	elseif ($returnColor = smf_snowflakes_hex_filter($color))
		return '#' . $returnColor;

	return '#0099FF';
}

function smf_snowflakes_permissions(&$permissionGroups, &$permissionList, &$leftPermissionGroups, &$hiddenPermissions, &$relabelPermissions)
{
	global $context, $smcFunc, $modSettings;
	loadLanguage('snowflakesSMF');

	$permissionList['membergroup'] += array(
		'snowflakes_no_view' => array(false, 'snowflakes_perms', 'snowflakes_perms'),
		'snowflakes_profile_view' => array(false, 'snowflakes_perms', 'snowflakes_perms'),
	);

	// SMF 2.1.X behavior will differ
	$version = version_compare((!empty($modSettings['smfVersion']) ? substr($modSettings['smfVersion'], 0, 3) : '2.0'), '2.1', '<') ? 'v2.0' : 'v2.1';

	if ($version === 'v2.0')
	{
		$permissionGroups['membergroup']['simple'] += array(
			'snowflakes_perms',
		);
		$permissionGroups['membergroup']['classic'] += array(
			'snowflakes_perms',
		);
	}
	else
		$permissionGroups['membergroup'] += array(
			'snowflakes_perms',
		);

	$context['non_guest_permissions'] = array_merge(
		$context['non_guest_permissions'],
		array(
			'snowflakes_profile_view',
		)
	);
}

function smf_snowflakes_images($current, $type)
{
	global $settings, $txt;
	loadLanguage('snowflakesSMF');

	list($imageTypeDefault, $imagePath) = array(array(), array());
	list($imagesetlist, $imageTypes, $imageTypeDefault[99], $imagePath[99]) = array(array(), array('gif', 'png', 'jpg', 'jpeg', 'bmp'), '', '');
	for ($a=0;$a<9;$a++)
	{
		$imageTypeDefault[$a] = $a == 8 ? 'wired_lights1.gif' : 'snowflake.png';
		$imagePath[$a] = $a == 8 ? 'wired_lights' : 'snowflakes';
	}

	if (!empty($imagePath[$type]) && is_dir($settings['default_theme_dir'] . '/images/' . $imagePath[$type] . '/'))
		$imagesDir = @opendir($settings['default_theme_dir'] . '/images/' . $imagePath[$type]);

	if (!empty($imagesDir))
	{
		while (($file = readdir($imagesDir)) !== false)
		{
			foreach ($imageTypes as $imageType)
				if (preg_match('#\.(?:' . $imageType . ')$#', $file))
				{
					$imagesetlist[] = $file;
					break;
				}
		}
		closedir($imagesDir);
	}

	natsort($imagesetlist);
	return array_merge(array($txt['smf_snowflakes_select']), $imagesetlist);
}

function smf_snowflakes_imageResize($src, $dst, $width, $height, $crop=0)
{
	global $txt;
	loadLanguage('snowflakesSMF');

	if(!list($w, $h) = getimagesize($src))
		return false;

	$type = strtolower(substr(strrchr($src,"."),1));
	if($type == 'jpeg')
		$type = 'jpg';

	switch($type)
	{
		case 'bmp':
			$img = @imagecreatefromwbmp($src);
			break;
		case 'gif':
			$img = @imagecreatefromgif($src);
			break;
		case 'jpg':
			$img = @imagecreatefromjpeg($src);
			break;
		case 'png':
			$img = @imagecreatefrompng($src);
			break;
		default:
			return false;
	}

	if (empty($img))
	{
		$file = array_reverse(explode('/', $src));
		$error = str_replace('@&!#', $file[0], $txt['snowflakesSMF_ErrorImage']);
		log_error($error, true);
		return false;
	}

	if($crop)
	{
		if($w < $width or $h < $height)
			return false;
		$ratio = max($width/$w, $height/$h);
		$h = $height / $ratio;
		$x = ($w - $width / $ratio) / 2;
		$w = $width / $ratio;
	}
	else
	{
		if($w < $width and $h < $height)
			return false;
		$ratio = min($width/$w, $height/$h);
		$width = $w * $ratio;
		$height = $h * $ratio;
		$x = 0;
	}

	$new = imagecreatetruecolor($width, $height);

	if($type == "gif" || $type == "png")
	{
		imagecolortransparent($new, imagecolorallocatealpha($new, 0, 0, 0, 127));
		imagealphablending($new, false);
		imagesavealpha($new, true);
	}

	imagecopyresampled($new, $img, 0, 0, $x, 0, $width, $height, $w, $h);

	$type = $type === 'gif' || $type === 'png' ? $type : 'ignore';
	switch($type)
	{
		case 'gif':
			imagegif($new, $dst);
			break;
		case 'png':
			imagepng($new, $dst);
			break;
		default:
			break;
	}

	return true;
}

function smf_snowflakes_defaults($snowflakesSettings)
{
	$type = !empty($snowflakesSettings['snowflakesType']) ? abs($snowflakesSettings['snowflakesType']) : 0;
	$images = smf_snowflakes_images($snowflakesSettings['snowflakesImage' . $type], $type);
	$key = array_search('snowflake.png', $images);
	$batkey = array_search('bats.png', $images);
	$snowWired = array_search('wired_lights1.gif', $images);

	$defaults = array(
		0 => array(
			'snowflakesNewOn0' => 1000,
			'snowflakesMinSize0' => 5,
			'snowflakesMaxSize0' => 50,
			'snowflakesFlakeColor0' => '#0099ff',
			'snowflakesDefaults' => 0,
		),
		1 => array(
			'snowflakesNewOn1' => 30,
			'snowflakesMinSize1' => 10,
			'snowflakesMaxSize1' => 20,
			'snowflakesFlakeColor1' => '#ffffff',
			'snowflakesMaxSpeed1' => 3,
			'snowflakesMinSpeed1' => 1,
			'snowflakesInterval1' => 50,
			'snowflakesFlakeCode1' => '&bull;',
			'snowflakesDefaults' => 0,
		),
		2 => array(
			'snowflakesNewOn2' => 35,
			'snowflakesMinSize2' => 3,
			'snowflakesMaxSize2' => 20,
			'snowflakesFlakeColor2' => '#ffffff',
			'snowflakesMaxSpeed2' => 5,
			'snowflakesMinSpeed2' => 1,
			'snowflakesImage2' => !empty($key) ? $key : 0,
			'snowflakesCollection2' => '',
			'snowflakesDefaults' => 0,
		),
		3 => array(
			'snowflakesNewOn3' => 64,
			'snowflakesStick3' => 'true',
			'snowflakesMelt3' => 'true',
			'snowflakesFlakeColor3' => '#ffffff',
			'snowflakesInterval3' => 33,
			'snowflakesMaxSpeed3' => 5,
			'snowflakesMinSpeed3' => 4,
			'snowflakesTwinkle3' => 'false',
			'snowflakesFlakeCode3' => '&bull;',
			'snowflakesFollow3' => 'true',
			'snowflakesMobile3' => 'true',
			'snowflakesWidth' => 8,
			'snowflakesHeight' => 8,
			'snowflakesDefaults' => 0,
		),
		4 => array(
			'snowflakesNewOn4' => 10,
			'snowflakesInterval4' => 300,
			'snowflakesMinSize4' => 10,
			'snowflakesMaxSize4' => 20,
			'snowflakesMinTime4' => 400,
			'snowflakesMaxTime4' => 1200,
			'snowflakesExplodeMaxTime4' => 800,
			'snowflakesExplodeMinTime4' => 400,
			'snowflakesFlakeCode4' => '&bull;',
			'snowflakesDefaults' => 0,
		),
		5 => array(
			'snowflakesNewOn5' => 5,
			'snowflakesInterval5' => 4,
			'snowflakesMaxSpeed5' => 20,
			'snowflakesFlicker5' => 15,
			'snowflakesImage5' => !empty($batkey) ? $batkey : 0,
			'snowflakesWidth5' => 35,
			'snowflakesHeight5' => 20,
			'snowflakesDefaults' => 0,
		),
		6 => array(
			'snowflakesAuto6' => 1,
			'snowflakesOffset6' => 2.7,
			'snowflakesContainerHeight6' => 4.5,
			'snowflakesDisableRange' => 210,
			'snowflakesIgnoreRange' => 1,
			'snowflakesDefaults' => 0,
		),
		7 => array(
			'snowflakesAuto7' => 1,
			'snowflakesOffset7' => 2.7,
			'snowflakesContainerHeight7' => 4.5,
			'snowflakesDisableRange' => 210,
			'snowflakesIgnoreRange' => 1,
			'snowflakesDefaults' => 0,
		),
		8 => array(
			'snowflakesAuto8' => 1,
			'snowflakesOffset8' => 2.7,
			'snowflakesContainerHeight8' => 5.5,
			'snowflakesDisableRange' => 210,
			'snowflakesDefaults' => 0,
			'snowflakesIgnoreRange' => 1,
			'snowflakesImage8' => !empty($snowWired) ? $snowWired : 0,
		),
		99 => array(
			'smf_snowflakes0' => 1,
			'smf_snowflakes1' => 1,
			'smf_snowflakes2' => 1,
			'smf_snowflakes3' => 1,
			'smf_snowflakes4' => 1,
			'smf_snowflakes5' => 1,
			'smf_snowflakes6' => 1,
			'smf_snowflakes7' => 1,
			'smf_snowflakes8' => 1,
		),
	);

	$_POST = array_replace($_POST, $defaults[$type]);
	$actions = isset($_POST['snowflakesActions']) ? explode(',', $_POST['snowflakesActions']) : array();
	$actions = array_filter($actions);
	$defaults = array('arcade', 'posts', 'downloads');
	$newActions = array_diff($actions, $defaults);
	foreach($defaults as $default) {
		if (!in_array($default, $newActions))
			$newActions[] = $default;
	}
	$_POST['snowflakesActions'] = implode(',', $newActions);
	$_FILES['file']['name'] = '';
	return $_POST;
}

function smf_snowflakes_profile(&$profile_areas)
{
	global $modSettings, $txt, $context, $settings;
	loadLanguage('snowflakesSMF');

	$profile_areas['edit_profile']['areas']+= array(
		'snowflakes' => array(
			'label' => $txt['snowflakes_profile'],
			'file' => 'snowflakesSMF.php',
			'icon' => 'configuration.png',
			'function' => 'smf_snowflakes_user_profile',
			'enabled' => true,
			'permission' => array(
				'own' => array('snowflakes_profile_view'),
				'any' => array('admin'),
			),
		),
	);
}

function smf_snowflakes_user_profile($memID)
{
	global $smcFunc, $sourcedir, $txt, $context, $modSettings, $user_info, $scripturl, $settings;
	loadLanguage('snowflakesSMF');
	require_once($sourcedir . '/Profile-Modify.php');

	loadThemeOptions($memID);

	if ($user_info['id'] != $memID && !$context['user']['is_admin'])
		redirectexit($scripturl . '?action=profile');
	elseif (!allowedTo(array('profile_extra_own')))
		redirectexit($scripturl . '?action=profile');

	$context['sub_template'] = 'edit_options';
	$context['page_desc'] = $txt['snowflakesEnable'];
	$help = '<a href="' . $scripturl . '?action=helpadmin;help=' . ($user_info['id'] == $memID ? 'snowflakes_prefs_help' : 'snowflakes_prefs_mod_help') . '" onclick="return reqWin(this.href);" style="text-decoration:none;"><img style="vertical-align: middle;position: relative;bottom: 2px;width: 12px;height: 12px;" src="' . $settings['default_theme_url'] . '/images/helptopics.png" alt="?" /></a>';
	if (isset($_POST['snowflakes_prefs']))
	{
		$val = !empty($_POST['snowflakes_prefs']) ? 1 : 0;
		$replaceArray[] = array($memID, $val);

		$smcFunc['db_query']('', '
			DELETE FROM {db_prefix}snowflakes_members
			WHERE user_id = {int:userid}',
			array('userid' => $memID,
			)
		);

		$smcFunc['db_insert']('insert',
			'{db_prefix}snowflakes_members',
			array('user_id' => 'int', 'disable' => 'int'),
			$replaceArray,
			array('user_id')
		);
	}

	$request = $smcFunc['db_query']('', '
		SELECT disable
		FROM {db_prefix}snowflakes_members
		WHERE user_id = {int:member}',
		array(
			'member' => $memID,
		)
	);
	list ($disabled) = $smcFunc['db_fetch_row']($request);
	$smcFunc['db_free_result'];

	$context['profile_fields'] = array(
		'snowflakes_prefs' => array(
			'type' => 'check',
			'label' => $help . '&nbsp;' . $txt['snowflakes_prefs'],
			'subtext' => '',
			'size' => 1,
			'input_attr' => '',
			'permission' => 'profile_own',
			'value' => !empty($disabled) ? 1 : 0,
		),
	);
}

function smf_snowflakes_prepareDBSettingContext(&$config_vars)
{
	global $txt, $helptxt, $context, $sourcedir;
	loadLanguage('snowflakesSMF');

	if (isset($_SESSION['adm-save']))
	{
		if ($_SESSION['adm-save'] === true)
			$context['saved_successful'] = true;
		else
			$context['saved_failed'] = $_SESSION['adm-save'];

		unset($_SESSION['adm-save']);
	}

	$snowflakesSettings = smf_snowflakes_getSettings();
	$context['config_vars'] = array();
	$inlinePermissions = array();
	foreach ($config_vars as $config_var)
	{
		// HR?
		if (!is_array($config_var))
			$context['config_vars'][] = $config_var;
		else
		{
			// If it has no name it doesn't have any purpose!
			if (empty($config_var[1]))
				continue;

			// Special case for inline permissions
			if ($config_var[0] == 'permissions' && allowedTo('manage_permissions'))
				$inlinePermissions[] = $config_var[1];
			elseif ($config_var[0] == 'permissions')
				continue;

			$context['config_vars'][$config_var[1]] = array(
				'label' => isset($config_var['text_label']) ? $config_var['text_label'] : (isset($txt[$config_var[1]]) ? $txt[$config_var[1]] : (isset($config_var[3]) && !is_array($config_var[3]) ? $config_var[3] : '')),
				'help' => isset($helptxt[$config_var[1]]) ? $config_var[1] : '',
				'type' => $config_var[0],
				'size' => !empty($config_var[2]) && !is_array($config_var[2]) ? $config_var[2] : (in_array($config_var[0], array('int', 'float')) ? 6 : 0),
				'data' => array(),
				'name' => $config_var[1],
				'value' => isset($snowflakesSettings[$config_var[1]]) ? ($config_var[0] == 'select' ? $snowflakesSettings[$config_var[1]] : htmlspecialchars($snowflakesSettings[$config_var[1]])) : (in_array($config_var[0], array('int', 'float')) ? 0 : ''),
				'disabled' => false,
				'invalid' => !empty($config_var['invalid']),
				'javascript' => '',
				'var_message' => !empty($config_var['message']) && isset($txt[$config_var['message']]) ? $txt[$config_var['message']] : '',
				'preinput' => isset($config_var['preinput']) ? $config_var['preinput'] : '',
				'postinput' => isset($config_var['postinput']) ? $config_var['postinput'] : '',
			);

			// If this is a select box handle any data.
			if (!empty($config_var[2]) && is_array($config_var[2]))
			{
				// If we allow multiple selections, we need to adjust a few things.
				if ($config_var[0] == 'select' && !empty($config_var['multiple']))
				{
					$context['config_vars'][$config_var[1]]['name'] .= '[]';
					$context['config_vars'][$config_var[1]]['value'] = !empty($context['config_vars'][$config_var[1]]['value']) ? unserialize($context['config_vars'][$config_var[1]]['value']) : array();
				}

				// If it's associative
				if (isset($config_var[2][0]) && is_array($config_var[2][0]))
					$context['config_vars'][$config_var[1]]['data'] = $config_var[2];
				else
				{
					foreach ($config_var[2] as $key => $item)
						$context['config_vars'][$config_var[1]]['data'][] = array($key, $item);
				}
			}

			// Finally allow overrides - and some final cleanups.
			foreach ($config_var as $k => $v)
			{
				if (!is_numeric($k))
				{
					if (substr($k, 0, 2) == 'on')
						$context['config_vars'][$config_var[1]]['javascript'] .= ' ' . $k . '="' . $v . '"';
					else
						$context['config_vars'][$config_var[1]][$k] = $v;
				}

				// See if there are any other labels that might fit?
				if (isset($txt['setting_' . $config_var[1]]))
					$context['config_vars'][$config_var[1]]['label'] = $txt['setting_' . $config_var[1]];
				elseif (isset($txt['groups_' . $config_var[1]]))
					$context['config_vars'][$config_var[1]]['label'] = $txt['groups_' . $config_var[1]];
			}

			// Set the subtext in case it's part of the label.
			$divPos = strpos($context['config_vars'][$config_var[1]]['label'], '<div');
			if ($divPos !== false)
			{
				$context['config_vars'][$config_var[1]]['subtext'] = preg_replace('~</?div[^>]*>~', '', substr($context['config_vars'][$config_var[1]]['label'], $divPos));
				$context['config_vars'][$config_var[1]]['label'] = substr($context['config_vars'][$config_var[1]]['label'], 0, $divPos);
			}
		}
	}

	// If we have inline permissions we need to prep them.
	if (!empty($inlinePermissions) && allowedTo('manage_permissions'))
	{
		require_once($sourcedir . '/ManagePermissions.php');
		init_inline_permissions($inlinePermissions, isset($context['permissions_excluded']) ? $context['permissions_excluded'] : array());
	}
}

function smf_snowflakes_saveDBSettings(&$config_vars)
{
	global $sourcedir, $context;

	if (!allowedTo('admin_forum'))
		fatal_lang_error('snowflakesSMF_ErrorMessage', false);

	$inlinePermissions = array();
	checkSession('post');

	foreach ($config_vars as $var)
	{
		if (!isset($var[1]) || (!isset($_POST[$var[1]]) && $var[0] != 'check' && $var[0] != 'permissions' && (!isset($_POST[$var[1] . '_enabledTags']))))
			continue;

		// Checkboxes!
		elseif ($var[0] == 'check')
			$setArray[$var[1]] = !empty($_POST[$var[1]]) ? '1' : '0';
		// Select boxes!
		elseif ($var[0] == 'select' && in_array($_POST[$var[1]], array_values($var[2])))
			$setArray[$var[1]] = $_POST[$var[1]];
		elseif ($var[0] == 'select' && in_array($_POST[$var[1]], array_keys($var[2])))
			$setArray[$var[1]] = $_POST[$var[1]];
		elseif ($var[0] == 'select' && !empty($var['multiple']) && array_intersect($_POST[$var[1]], array_keys($var[2])) != array())
		{
			// For security purposes we validate this line by line.
			$options = array();
			foreach ($_POST[$var[1]] as $invar)
				if (in_array($invar, array_keys($var[2])))
					$options[] = $invar;

			$setArray[$var[1]] = serialize($options);
		}
		// Integers!
		elseif ($var[0] == 'int')
			$setArray[$var[1]] = (int) $_POST[$var[1]];
		// Floating point!
		elseif ($var[0] == 'float')
			$setArray[$var[1]] = (float) $_POST[$var[1]];
		// Text!
		elseif ($var[0] == 'text' || $var[0] == 'large_text')
			$setArray[$var[1]] = $_POST[$var[1]];
		// Passwords!
		elseif ($var[0] == 'password')
		{
			if (isset($_POST[$var[1]][1]) && $_POST[$var[1]][0] == $_POST[$var[1]][1])
				$setArray[$var[1]] = $_POST[$var[1]][0];
		}
		// Permissions?
		elseif ($var[0] == 'permissions')
			$inlinePermissions[] = $var[1];
	}

	if (!empty($setArray))
		smf_snowflakes_updateSettings($setArray);

	// If we have inline permissions we need to save them.
	if (!empty($inlinePermissions) && allowedTo('manage_permissions'))
		smf_snowflakes_inline_permissions($inlinePermissions);

}

function smf_snowflakes_updateSettings($changeArray, $update = false, $debug = false)
{
	global $smcFunc;

	if (empty($changeArray) || !is_array($changeArray))
		return;

	$snowflakesSettings = smf_snowflakes_getSettings();

	if ($update)
	{
		foreach ($changeArray as $variable => $value)
		{
			$smcFunc['db_query']('', '
				UPDATE {db_prefix}snowflakes_settings
				SET value = {' . ($value === false || $value === true ? 'raw' : 'string') . ':value}
				WHERE variable = {string:variable}',
				array(
					'value' => $value === true ? 'value + 1' : ($value === false ? 'value - 1' : $value),
					'variable' => $variable,
				)
			);
			$snowflakesSettings[$variable] = $value === true ? $snowflakesSettings[$variable] + 1 : ($value === false ? $snowflakesSettings[$variable] - 1 : $value);
		}

		cache_put_data('snowflakesSettings', null, 90);
		return;
	}

	$replaceArray = array();
	foreach ($changeArray as $variable => $value)
	{
		if (isset($snowflakesSettings[$variable]) && $snowflakesSettings[$variable] == $value)
			continue;
		elseif (!isset($snowflakesSettings[$variable]) && empty($value))
			continue;

		$replaceArray[] = array($variable, $value);

		$snowflakesSettings[$variable] = $value;
	}

	if (empty($replaceArray))
		return;

	$smcFunc['db_insert']('replace',
		'{db_prefix}snowflakes_settings',
		array('variable' => 'string-255', 'value' => 'string-65534'),
		$replaceArray,
		array('variable')
	);

	cache_put_data('snowflakesSettings', null, 90);
}

function smf_snowflakes_getSettings($spec = '99')
{
	global $smcFunc, $modSettings, $sourcedir, $snowflakesSettings;

	$spec = floatval($spec);
	if (!defined("SNOW_FLAKES_IS_MOBILE") || !defined("SNOW_FLAKES_IS_TABLET"))
		smf_snowflakes_helpText();
	if ($spec >= 0 && $spec != 99)
	{
		$request = $smcFunc['db_query']('', "
			SELECT variable, value
			FROM {db_prefix}snowflakes_settings
			WHERE variable LIKE 'snowflakes%" . (string)$spec . "' OR variable NOT REGEXP '[0-9]+$'",
			array(
			)
		);

		$snowflakesSettings = array();
		if (!$request)
			db_fatal_error();
		while ($row = $smcFunc['db_fetch_row']($request))
			$snowflakesSettings[$row[0]] = $row[1];
		$smcFunc['db_free_result']($request);

		$snowflakesSettings['isMobile'] = SNOW_FLAKES_IS_MOBILE;
		$snowflakesSettings['isTablet'] = SNOW_FLAKES_IS_TABLET;

		return $snowflakesSettings;
	}

	if (($snowflakesSettings = cache_get_data('snowflakesSettings', 90)) == null)
	{
		$request = $smcFunc['db_query']('', '
			SELECT variable, value
			FROM {db_prefix}snowflakes_settings',
			array(
			)
		);

		$snowflakesSettings = array();
		if (!$request)
			db_fatal_error();
		while ($row = $smcFunc['db_fetch_row']($request))
			$snowflakesSettings[$row[0]] = $row[1];
		$smcFunc['db_free_result']($request);

		if (!empty($modSettings['cache_enable']))
			cache_put_data('snowflakesSettings', $snowflakesSettings, 90);
	}

	$snowflakesSettings['isMobile'] = SNOW_FLAKES_IS_MOBILE;
	$snowflakesSettings['isTablet'] = SNOW_FLAKES_IS_TABLET;
	$snowflakesSettings['snowflakesType'] = !empty($snowflakesSettings['snowflakesType']) ? (int)$snowflakesSettings['snowflakesType'] : 0;
	return $snowflakesSettings;
}

function smf_snowflakes_helpText()
{
	global $helptxt, $sourcedir, $txt;
	loadLanguage('snowflakesSMF_Help');
	require_once($sourcedir . '/snowflakesMobileDetect.php');
	$detect = new Snowflakes_Mobile_Detect;

	if (!defined("SNOW_FLAKES_IS_MOBILE"))
		define("SNOW_FLAKES_IS_MOBILE", $detect->isSnowMobile());

	if (!defined("SNOW_FLAKES_IS_TABLET"))
		define("SNOW_FLAKES_IS_TABLET", $detect->isSnowTablet());
}

function smf_snowflakes_dom()
{
	global $settings;

	$output = '
	<script type="text/javascript"><!-- // --><![CDATA[
		var snowInterval;
		var snowIntervalCount=0;
		snowInterval = setInterval(function(){
			if (typeof smfSnowflakesBulbs === "function") {
				clearInterval(snowInterval);
				console.log("lights script loaded");
			}
			else if (snowIntervalCount >= 50) {
				clearInterval(snowInterval);
				console.log("lights script failed after 5 seconds");
			}
			else {
				console.log("waiting for lights script to load...");
				$.getScript("' . $settings['default_theme_url'] . '/scripts/jquery.snow6.js");
				snowIntervalCount++;			}
		}, 100);
	// ]]></script>';
	// document.body.appendChild(snowScriptId6);
	return $output;
}

function smf_snowflakes_inline_permissions($permissions)
{
	global $context, $smcFunc, $sourcedir;

	if (!allowedTo('admin_forum'))
		return;

	require_once($sourcedir . '/ManagePermissions.php');
	loadIllegalPermissions();
	$insertRows = array();
	foreach ($permissions as $permission)
	{
		if (isset($_POST[$permission]))
		{
			$_POST[$permission] = !is_array($_POST[$permission]) ? array($_POST[$permission]) : $_POST[$permission];
			foreach ($_POST[$permission] as $id_group => $value)
			{
				if (in_array($value, array('on', 'deny')) && (empty($context['illegal_permissions']) || !in_array($permission, $context['illegal_permissions'])))
					$insertRows[] = array((int) $id_group, $permission, $value == 'on' ? 1 : 0);
			}
		}
	}

	$smcFunc['db_query']('', '
		DELETE FROM {db_prefix}permissions
		WHERE permission IN ({array_string:permissions})
			' . (empty($context['illegal_permissions']) ? '' : ' AND permission NOT IN ({array_string:illegal_permissions})'),
		array(
			'illegal_permissions' => !empty($context['illegal_permissions']) ? $context['illegal_permissions'] : array(),
			'permissions' => $permissions,
		)
	);

	if (!empty($insertRows))
		$smcFunc['db_insert']('insert',
			'{db_prefix}permissions',
			array('id_group' => 'int', 'permission' => 'string', 'add_deny' => 'int'),
			$insertRows,
			array('id_group', 'permission')
		);

	updateChildPermissions(array(), -1);
	updateSettings(array('settings_updated' => time()));
}

?>