<?php
/*
*	<id>chenzhen:snowflakesSMF</id>
*	<name>Snowflakes</name>
*	<type>modification</type>
*/

/*
 * Snowflakes display Modification for SMF forums c/o Chen Zhen @ https://web-develop.ca
 * Copyright 2014 - 2023
 * This software package is distributed under the GNU Lesser General Public License version 3 ( https://opensource.org/licenses/lgpl-3.0.html )
*/

// Use this file by using SSI.php
if (file_exists(dirname(__FILE__) . '/SSI.php') && !defined('SMF'))
    require_once(dirname(__FILE__) . '/SSI.php');
elseif (!defined('SMF'))
    die('<b>Error:</b> Cannot install - please verify you put this in the same place as SMF\'s SSI.php.');

// determine SMF version
global $smcFunc, $modSettings;
db_extend('packages');
$version = version_compare((!empty($modSettings['smfVersion']) ? substr($modSettings['smfVersion'], 0, 3) : '2.0'), '2.1', '<') ? 'v2' : 'v2.1';
$snowflakesVersion = 'v1.83';

/* Insert integration hooks */
if ($version === 'v2.1')
{
    add_integration_function('integrate_buffer', '$sourcedir/snowflakesSMF.php|smf_snowflakes_buffer');
    add_integration_function('integrate_modify_modifications', '$sourcedir/snowflakesSMF.php|smf_snowflakes_mod');
    add_integration_function('integrate_admin_areas', '$sourcedir/snowflakesSMF.php|smf_snowflakes_admin');
    add_integration_function('integrate_load_permissions', '$sourcedir/snowflakesSMF.php|smf_snowflakes_permissions');
    add_integration_function('integrate_pre_profile_areas', '$sourcedir/snowflakesSMF.php|smf_snowflakes_profile');
    add_integration_function('integrate_pre_load', '$sourcedir/snowflakesSMF.php|smf_snowflakes_helpText');
}
else
{
    add_integration_function('integrate_pre_include', '$sourcedir/snowflakesSMF.php');
    add_integration_function('integrate_modify_modifications', 'smf_snowflakes_mod');
    add_integration_function('integrate_admin_areas', 'smf_snowflakes_admin');
    add_integration_function('integrate_load_permissions', 'smf_snowflakes_permissions');
    add_integration_function('integrate_profile_areas', 'smf_snowflakes_profile');
    add_integration_function('integrate_pre_load', 'smf_snowflakes_helpText');
	add_integration_function('integrate_buffer', 'smf_snowflakes_buffer');
	add_integration_function('integrate_load_theme', 'smf_snowflakes_post_preview');
}

// create snowflakes_settings table if necessary
$columns = array(
    array(
	'name' => 'variable',
	'type' => 'varchar',
	'size' => 255,
	'null' => false,
	'auto' => false,
	'default' => '',
    ),
    array(
	'name' => 'value',
	'type' => 'text',
	'null' => false,
    ),
);
$indexes = array(
	array(
	    'type' => 'primary',
	    'columns' => array('variable(30)'),
	    'engine' => 'MyISAM'
	)
);
$smcFunc['db_create_table'] ('{db_prefix}snowflakes_settings', $columns, $indexes, array(), 'ignore');

$user_columns = array(
	array(
	    'name' => 'user_id',
	    'type' => 'mediumint',
	    'size' => 8,
	    'null' => false,
	    'unsigned' => true,
	    'auto' => false,
	),
	array(
	    'name' => 'disable',
	    'type' => 'int',
	    'size' => 10,
	    'null' => false,
	    'unsigned' => true,
	    'auto' => false,
	),
);
$user_indexes = array(
	array(
	    'type' => 'primary',
	    'columns' => array('user_id')
	),
);
$smcFunc['db_create_table'] ('{db_prefix}snowflakes_members', $user_columns, $user_indexes, array(), 'ignore');

// add defaults if no data exists
$snowflakesSettings = smf_snowflakes_installSettings();
$defaults = array(
    'snowflakesAppend' => 0,
    'snowflakesMinSize0' => 5,
    'snowflakesMaxSize0' => 50,
    'snowflakesFlakeColor0' => '#0099ff',
    'snowflakesNewOn0' => 1000,
    'snowflakesType' => 0,
    'snowflakesTwinkle3' => 1,
    'snowflakesFollow3' => 1,
    'snowflakesMelt3' => 1,
    'snowflakesStick3' => 1,
    'snowflakesMobile3' => 1,
    'snowflakesFlakeColor3' => '#ffffff',
    'snowflakesNewOn3' => 64,
    'snowflakesMinSpeed3' => 4,
    'snowflakesMaxSpeed3' => 5,
    'snowflakesFlakeCode3' => '&bull;',
    'snowflakesEnable' => 1,
    'snowflakesFiles' => 1,
    'snowflakesAdminArea' => 1,
    'snowflakesMinSize1' => 10,
    'snowflakesMaxSize1' => 20,
    'snowflakesFlakeColor1' => '#ffffff',
    'snowflakesInterval1' => 50,
    'snowflakesNewOn1' => 30,
    'snowflakesMinSpeed1' => 1,
    'snowflakesMaxSpeed1' => 3,
    'snowflakesFlakeCode1' => '&bull;',
    'snowflakesMinSize2' => 3,
    'snowflakesMaxSize2' => 20,
    'snowflakesMinSpeed2' => 1,
    'snowflakesMaxSpeed2' => 5,
    'snowflakesNewOn2' => 35,
    'snowflakesMinSize4' => 10,
    'snowflakesMaxSize4' => 20,
    'snowflakesNewOn4' => 10,
    'snowflakesInterval4' => 300,
    'snowflakesMinTime4' => 400,
    'snowflakesMaxTime4' => 1200,
    'snowflakesExplodeMinTime4' => 400,
    'snowflakesExplodeMaxTime4' => 800,
    'snowflakesFlakeCode4' => '&bull;',
    'snowflakesNewOn5' => 5,
    'snowflakesInterval5' => 4,
    'snowflakesMaxSpeed5' => 20,
    'snowflakesFlicker5' => 15,
    'snowflakesWidth5' => 35,
    'snowflakesHeight5' => 20,
    'snowflakesImage' => 17,
    'snowflakesImage5' => 'bats.png',
    'snowflakesOffset6' => 0,
	'snowflakesContainerHeight6' => 4.5,
    'snowflakesAuto6' => 1,
    'snowflakesImage2' => 'snowflake.png',
	'snowflakesDisableRange' => 210,
	'snowflakesIgnoreRange' => 1,
);

//$snowflakesSettings = array_replace($snowflakesSettings, $defaults);
foreach ($defaults as $variable => $value)
{
    if (!isset($snowflakesSettings[$variable]))
		$replaceArray[] = array($variable, $value);
}
$replaceArray[] = array('snowflakesVersion', $snowflakesVersion);

if (!empty($replaceArray))
    $smcFunc['db_insert']('replace', '{db_prefix}snowflakes_settings',
		array('variable' => 'string-255', 'value' => 'string-65534'),
		$replaceArray,
		array('variable')
    );



// query data from the custom snowflakes_settings table
function smf_snowflakes_installSettings()
{
	global $smcFunc;

	$request = $smcFunc['db_query']('', '
		SELECT variable, value
		FROM {db_prefix}snowflakes_settings',
		array()
	);
	$snowflakesSettings = array();
	if (!$request)
		db_fatal_error();

	while ($row = $smcFunc['db_fetch_row']($request))
		$snowflakesSettings[$row[0]] = $row[1];

	$smcFunc['db_free_result']($request);

	return $snowflakesSettings;
}
?>